<template>
  <b-card-code>
    <b-form @submit.prevent="onSubmit">
      <!-- 全優惠模組共通表單欄位組件 -->
      <promotion-base-form
        :promotion="promotion"
        :strErrors="strErrors"
        @initChannels="initChannels"
      ></promotion-base-form>
      <!-- 全優惠模組共通表單欄位組件 End -->
      <b-row>
        <b-col cols="12">
          <b-form-group
            label="*加購門檻"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-col class="col-md-2 d-inline-block px-0">
              <b-form-input
                type="number"
                v-model="promotion.condition_value"
                :placeholder="
                  promotion.condition_type == 1 ? '請輸入金額' : '請輸入件數'
                "
              />
            </b-col>

            <b-col class="col-lg-2 col-xl-1 d-inline-block px-0">
              <v-select
                v-model="promotion.condition_type"
                :options="conditionTypeOptions"
                :clearable="false"
                :reduce="(conditionType) => conditionType.id"
                label="name"
                class="item-selector-title"
              />
            </b-col>
          </b-form-group>
        </b-col>

        <b-col cols="12">
          <b-form-group
            label="加購品"
            label-cols-md="1"
            class="text-left"
            label-size="lg"
          >
            <b-button
              class="d-inline-block"
              label-cols-md="1"
              variant="primary"
              @click="addProduct"
            >
              增加加購品
            </b-button>
          </b-form-group>
        </b-col>

        <b-col cols="11" offset-md="1">
          <div ref="form" class="repeater-form">
            <div v-for="(product, index) in promotion.products" :key="index">
              <b-row class="d-flex rounded mb-1">
                <b-col cols="12" lg="3">
                  <label class="d-inline">商品</label>
                  <v-select
                    v-model="product.product_id"
                    :options="products"
                    :clearable="false"
                    :reduce="(product) => product.id"
                    label="name_frontend"
                    class="item-selector-title"
                  />
                </b-col>

                <b-col cols="12" lg="1">
                  <label class="d-inline">加購價</label>
                  <div class="mb-1">
                    <b-form-input v-model="product.price" type="number" />
                    <small
                      >原價：{{
                        getProductInfo(product.product_id).price
                      }}</small
                    >
                  </div>
                </b-col>

                <b-col
                  cols="12"
                  lg="1"
                  class="pr-0"
                  v-for="channel in product.channels"
                  :key="channel.channel_id"
                >
                  <label class="d-inline"
                    >{{ channel.channel_name }}可購量</label
                  >
                  <div class="mb-1">
                    <b-form-input
                      v-model="channel.sales_limit"
                      type="number"
                      min="0"
                    />
                  </div>
                </b-col>

                <b-col cols="12" lg="2">
                  <label class="d-inline">限購數量</label>
                  <div class="mb-1">
                    <b-input-group>
                      <b-input-group-prepend is-text>
                        <b-form-checkbox
                          class="custom-control-primary"
                          :checked="product.purchase_limit == null"
                          @change="toggleIsPurchaseLimit($event, product)"
                        >
                          無限制
                        </b-form-checkbox>
                      </b-input-group-prepend>

                      <b-form-input
                        v-model="product.purchase_limit"
                        type="number"
                        :disabled="product.purchase_limit == null"
                        min="0"
                      />
                    </b-input-group>
                  </div>
                </b-col>

                <b-col cols="12" lg="auto">
                  <label class="d-inline">是否啟用</label>
                  <div style="height: 32px; padding-top: 6px">
                    <b-form-checkbox
                      v-model="product.enabled"
                      checked="true"
                      class="custom-control-primary"
                      name="check-button"
                      switch
                    />
                  </div>
                </b-col>

                <b-col cols="12" lg="auto">
                  <label class="d-inline">刪除</label>
                  <div>
                    <feather-icon
                      size="32"
                      icon="XIcon"
                      class="cursor-pointer text-danger"
                      @click="removeProduct(index)"
                    />
                  </div>
                </b-col>
              </b-row>
            </div>
          </div>
        </b-col>

        <!-- submit and reset -->
      </b-row>

      <b-row align-h="end">
        <b-col cols="auto">
          <b-button
            class="float-right"
            v-ripple.400="'rgba(255, 255, 255, 0.15)'"
            type="submit"
            variant="primary"
          >
            {{ promotion.id ? "儲存" : "新增" }}
          </b-button>
        </b-col>
      </b-row>
    </b-form>
  </b-card-code>
</template>

<script>
import BCardCode from "@core/components/b-card-code";
import Ripple from "vue-ripple-directive";
import vSelect from "vue-select";
import store from "@/store";
import { onUnmounted, ref } from "@vue/composition-api";
import router from "@/router";
import PromotionBaseForm from "../PromotionBaseForm.vue";
import promotionModule from "../promotionModule";

export default {
  components: {
    BCardCode,
    vSelect,
    PromotionBaseForm,
  },
  directives: {
    Ripple,
  },
  watch: {
    "promotion.apply_channels": {
      handler: function (apply_channels) {
        this.channels.forEach((channel) => {
          let isSelected = apply_channels.some(
            (apply_channel) => apply_channel.id == channel.id
          );

          this.promotion.products.forEach((product) => {
            let has = product.channels.some(
              (productChannel) => productChannel.channel_id == channel.id
            );

            // 新增promotion.channels
            if (isSelected && !has) {
              product.channels.push({
                channel_id: channel.id,
                channel_name: channel.name,
                used_limit: 0,
              });
            }

            // 移除promotion.channels
            if (!isSelected && has) {
              product.channels = product.channels.filter(
                (productChannel) => productChannel.channel_id != channel.id
              );
            }

            // 排序
            product.channels.sort((a, b) => a.channel_id - b.channel_id);
          });
        });
      },
    },
  },
  mounted() {
    if (router.currentRoute.params.id) {
      this.getPromotion();
    }

    this.getProducts();
  },
  methods: {
    // 刪除贈品
    removeProduct(index) {
      this.promotion.products.splice(index, 1);
    },

    // 刪除條件
    removeCondition(index) {
      this.promotion.conditions.splice(index, 1);
    },

    // 增加贈品
    addProduct() {
      let productChannels = [];

      this.promotion.apply_channels.forEach((channel) => {
        productChannels.push({
          channel_id: channel.id,
          channel_name: channel.name,
          sales_limit: null,
        });
      });

      this.promotion.products.push({
        product_id: null,
        price: null,
        purchase_limit: null,
        channels: productChannels,
        enabled: true,
      });
    },

    // 切換是否限購
    toggleIsPurchaseLimit(changed, product) {
      product.purchase_limit = changed ? null : 1;
    },

    // 取得商品資訊
    getProductInfo(id) {
      let product = this.products.find((product) => product.id == id);

      return (
        product || {
          id: null,
          spu: null,
          name_frontend: null,
          stock: null,
          MOQ: null,
          MPN: null,
          products_class_main_id: null,
          products_class_sub_id: null,
          price: null,
        }
      );
    },

    // 增加條件
    addCondition() {
      this.promotion.conditions.push({
        type: null,
        value: null,
        is_accumulate: true,
        enabled: true,
      });
    },

    // 新增資料
    onSubmit() {
      if (router.currentRoute.params.id) {
        this.updatePromotion();
      } else {
        this.createPromotion();
      }
    },

    // 新增優惠
    createPromotion() {
      store
        .dispatch("promotions/createPromotion", {
          typeName: this.TYPE_NAME,
          promotion: this.promotion,
        })
        .then((response) => {
          let result = response.data;

          // 錯誤處理
          if (result.success === false) {
            this.errorMessage(result.message);

            return false;
          }

          this.successMessage("新增成功");

          router.push({ name: "promotions-additions-list" });
        })
        .catch((error) => {
          let result = error.response;

          if (result.status === 422 && result?.data?.errors) {
            this.strErrors = result.data.errors;
          }

          this.errorMessage("新增失敗：" + result?.data?.message);
        });
    },

    // 更新優惠
    updatePromotion() {
      store
        .dispatch("promotions/updatePromotion", {
          typeName: this.TYPE_NAME,
          promotion: this.promotion,
        })
        .then((response) => {
          let result = response.data;

          // 錯誤處理
          if (result.success === false) {
            this.errorMessage(result.message);

            return false;
          }

          this.successMessage("更新成功");

          router.push({ name: "promotions-additions-list" });
        })
        .catch((error) => {
          let result = error.response;

          if (result.status === 422 && result?.data?.errors) {
            this.strErrors = result.data.errors;
          }

          this.errorMessage("更新失敗：" + result?.data?.message);
        });
    },

    // 取得優惠
    getPromotion() {
      store
        .dispatch("promotions/getPromotion", {
          typeName: this.TYPE_NAME,
          id: router.currentRoute.params.id,
        })
        .then((response) => {
          if (response.data?.success === false) {
            this.errorMessage(response.data?.message);

            return false;
          }

          this.promotion = response.data?.data;
        })
        .catch((error) => {
          this.errorMessage(error.response?.data?.message);
        });
    },

    // 取得產品清單
    getProducts() {
      store
        .dispatch("promotions/getProducts", {})
        .then((response) => {
          if (response.data?.success === false) {
            this.errorMessage(response.data?.message);

            return false;
          }

          this.products = response.data?.data;
        })
        .catch((error) => {
          this.errorMessage(error.response?.data?.message);
        });
    },

    // 取得渠道
    initChannels(channels = []) {
      this.channels = channels;
    },

    // 顯示成功視窗
    successMessage(text = null, title = "成功") {
      this.$swal({
        title: title,
        text: text,
        icon: "success",
        customClass: {
          confirmButton: "btn btn-primary",
        },
        buttonsStyling: false,
      });
    },

    // 顯示錯誤視窗
    errorMessage(text = null, title = "錯誤") {
      this.$swal({
        title: title,
        text: text,
        icon: "error",
        customClass: {
          confirmButton: "btn btn-primary",
        },
        buttonsStyling: false,
      });
    },
  },
  setup() {
    const USER_APP_STORE_MODULE_NAME = "promotions";

    // Register module
    if (!store.hasModule(USER_APP_STORE_MODULE_NAME))
      store.registerModule(USER_APP_STORE_MODULE_NAME, promotionModule);

    // UnRegister on leave
    onUnmounted(() => {
      if (store.hasModule(USER_APP_STORE_MODULE_NAME))
        store.unregisterModule(USER_APP_STORE_MODULE_NAME);
    });

    // 優惠類型
    const TYPE_NAME = "additions";

    // 門檻值單位
    let conditionTypeOptions = ref([
      { id: 1, name: "元" },
      { id: 2, name: "件" },
    ]);

    let promotion = ref({
      id: null,
      name: null,
      start_at: null,
      end_at: null,
      description: null,
      excepts: [],
      apply_channels: [],
      type: 4, // 預設優惠類型，很重要，影響到新增
      mode: 1,
      condition_type: 1,
      condition_value: null,
      products: [],
    });

    let strErrors = ref({
      name: null,
      start_at: null,
      end_at: null,
      description: null,
      excepts: null,
      apply_channels: null,
      type: null,
      mode: null,
      condition_type: null,
      condition_value: null,
      products: null,
    });

    let channels = ref([]);

    let products = ref([]);

    return {
      TYPE_NAME,
      conditionTypeOptions,
      promotion,
      strErrors,
      channels,
      products,
    };
  },
};
</script>
<style lang="scss">
@import "~@resources/scss/vue/libs/quill.scss";
@import "~@resources/scss/vue/libs/vue-sweetalert.scss";
@import "~@resources/scss/vue/libs/vue-select.scss";
</style>